@@ -3,7 +3,7 @@ import shutil
 import tempfile
 
 from mock import patch
-from mock import Mock
+from mock import Mock, MagicMock
 
 from zope.interface import implements
 
@@ -39,9 +39,6 @@ from txtorcon.endpoints import default_tcp4_endpoint_generator
 import util
 
 
-connectionRefusedFailure = Failure(ConnectionRefusedError())
-
-
 class EndpointTests(unittest.TestCase):
 
     def setUp(self):
@@ -254,19 +251,24 @@ class EndpointTests(unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_explicit_data_dir(self):
-        config = TorConfig(self.protocol)
-        ep = TCPHiddenServiceEndpoint(self.reactor, config, 123, '/dev/null')
+        d = tempfile.mkdtemp()
+        try:
+            with open(os.path.join(d, 'hostname'), 'w') as f:
+                f.write('public')
 
-        # make sure listen() correctly configures our hidden-serivce
-        # with the explicit directory we passed in above
-        d = ep.listen(NoOpProtocolFactory())
+            config = TorConfig(self.protocol)
+            ep = TCPHiddenServiceEndpoint(self.reactor, config, 123, d)
 
-        def foo(fail):
-            print "ERROR", fail
-        d.addErrback(foo)
-        port = yield d
-        self.assertEqual(1, len(config.HiddenServices))
-        self.assertEqual(config.HiddenServices[0].dir, '/dev/null')
+            # make sure listen() correctly configures our hidden-serivce
+            # with the explicit directory we passed in above
+            port = yield ep.listen(NoOpProtocolFactory())
+
+            self.assertEqual(1, len(config.HiddenServices))
+            self.assertEqual(config.HiddenServices[0].dir, d)
+            self.assertEqual(config.HiddenServices[0].hostname, 'public')
+
+        finally:
+            shutil.rmtree(d, ignore_errors=True)
 
     def test_failure(self):
         self.reactor.failures = 1
@@ -361,6 +363,32 @@ class EndpointTests(unittest.TestCase):
         finally:
             os.chdir(orig)
 
+    @defer.inlineCallbacks
+    def test_stealth_auth(self):
+        '''
+        make sure we produce a HiddenService instance with stealth-auth
+        lines if we had authentication specified in the first place.
+        '''
+
+        config = TorConfig(self.protocol)
+        ep = TCPHiddenServiceEndpoint(self.reactor, config, 123, '/dev/null',
+                                      stealth_auth=['alice', 'bob'])
+
+        # make sure listen() correctly configures our hidden-serivce
+        # with the explicit directory we passed in above
+        d = ep.listen(NoOpProtocolFactory())
+
+        def foo(fail):
+            print "ERROR", fail
+        d.addErrback(foo)
+        port = yield d
+        self.assertEqual(1, len(config.HiddenServices))
+        self.assertEqual(config.HiddenServices[0].dir, '/dev/null')
+        self.assertEqual(config.HiddenServices[0].authorize_client[0], 'stealth alice,bob')
+        self.assertEqual(None, ep.onion_uri)
+        config.HiddenServices[0].hostname = 'oh my'
+        self.assertEqual('oh my', ep.onion_uri)
+
 
 class EndpointLaunchTests(unittest.TestCase):
 
@@ -540,12 +568,12 @@ class FakeTorSocksEndpoint(object):
         self.transport = None
 
         self.failure = kw.get('failure', None)
-        self.acceptPort = kw.get('acceptPort', None)
+        self.accept_port = kw.get('accept_port', None)
 
     def connect(self, fac):
         self.factory = fac
-        if self.acceptPort:
-            if self.port != self.acceptPort:
+        if self.accept_port:
+            if self.port != self.accept_port:
                 return defer.fail(self.failure)
         else:
             if self.failure:
@@ -564,10 +592,10 @@ class TestTorClientEndpoint(unittest.TestCase):
         This test is equivalent to txsocksx's
         TestSOCKS4ClientEndpoint.test_clientConnectionFailed
         """
-        def FailTorSocksEndpointGenerator(*args, **kw):
-            kw['failure'] = connectionRefusedFailure
+        def fail_tor_socks_endpoint_generator(*args, **kw):
+            kw['failure'] = Failure(ConnectionRefusedError())
             return FakeTorSocksEndpoint(*args, **kw)
-        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=FailTorSocksEndpointGenerator)
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=fail_tor_socks_endpoint_generator)
         d = endpoint.connect(None)
         return self.assertFailure(d, ConnectionRefusedError)
 
@@ -575,18 +603,18 @@ class TestTorClientEndpoint(unittest.TestCase):
         """
         Same as above, but with a username/password.
         """
-        def FailTorSocksEndpointGenerator(*args, **kw):
-            kw['failure'] = connectionRefusedFailure
+        def fail_tor_socks_endpoint_generator(*args, **kw):
+            kw['failure'] = Failure(ConnectionRefusedError())
             return FakeTorSocksEndpoint(*args, **kw)
         endpoint = TorClientEndpoint(
             'invalid host', 0,
             socks_username='billy', socks_password='s333cure',
-            _proxy_endpoint_generator=FailTorSocksEndpointGenerator)
+            _proxy_endpoint_generator=fail_tor_socks_endpoint_generator)
         d = endpoint.connect(None)
         return self.assertFailure(d, ConnectionRefusedError)
 
     def test_default_generator(self):
-        # just ensuring the default generator doesn't blow updoesn't blow up
+        # just ensuring the default generator doesn't blow up
         default_tcp4_endpoint_generator(None, 'foo.bar', 1234)
 
     def test_no_host(self):
@@ -616,28 +644,49 @@ class TestTorClientEndpoint(unittest.TestCase):
         """
         This test is equivalent to txsocksx's TestSOCKS5ClientEndpoint.test_defaultFactory
         """
-        def TorSocksEndpointGenerator(*args, **kw):
+        endpoints = []
+
+        def tor_socks_endpoint_generator(*args, **kw):
+            endpoints.append(FakeTorSocksEndpoint(*args, **kw))
+            return endpoints[-1]
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=tor_socks_endpoint_generator)
+        endpoint.connect(Mock)
+        self.assertEqual(1, len(endpoints))
+        self.assertEqual(endpoints[0].transport.value(), '\x05\x01\x00')
+
+    @patch('txtorcon.endpoints.SOCKS5ClientEndpoint')
+    @defer.inlineCallbacks
+    def test_success(self, socks5_factory):
+        ep = MagicMock()
+        gold_proto = object()
+        ep.connect = MagicMock(return_value=gold_proto)
+        socks5_factory.return_value = ep
+
+        def tor_socks_endpoint_generator(*args, **kw):
             return FakeTorSocksEndpoint(*args, **kw)
-        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator)
-        endpoint.connect(None)
-        self.assertEqual(endpoint.tor_socks_endpoint.transport.value(), '\x05\x01\x00')
+
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=tor_socks_endpoint_generator)
+        other_proto = yield endpoint.connect(MagicMock())
+        self.assertEqual(other_proto, gold_proto)
 
     def test_good_port_retry(self):
         """
         This tests that our Tor client endpoint retry logic works correctly.
-        We create a proxy endpoint that fires a connectionRefusedFailure
+        We create a proxy endpoint that fires a ConnectionRefusedError
         unless the connecting port matches. We attempt to connect with the
         proxy endpoint for each port that the Tor client endpoint will try.
         """
         success_ports = TorClientEndpoint.socks_ports_to_try
+        endpoints = []
         for port in success_ports:
-            def TorSocksEndpointGenerator(*args, **kw):
-                kw['acceptPort'] = port
-                kw['failure'] = connectionRefusedFailure
-                return FakeTorSocksEndpoint(*args, **kw)
-            endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator)
+            def tor_socks_endpoint_generator(*args, **kw):
+                kw['accept_port'] = port
+                kw['failure'] = Failure(ConnectionRefusedError())
+                endpoints.append(FakeTorSocksEndpoint(*args, **kw))
+                return endpoints[-1]
+            endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=tor_socks_endpoint_generator)
             endpoint.connect(None)
-            self.assertEqual(endpoint.tor_socks_endpoint.transport.value(), '\x05\x01\x00')
+            self.assertEqual(endpoints[-1].transport.value(), '\x05\x01\x00')
 
     def test_bad_port_retry(self):
         """
@@ -645,11 +694,11 @@ class TestTorClientEndpoint(unittest.TestCase):
         """
         fail_ports = [1984, 666]
         for port in fail_ports:
-            def TorSocksEndpointGenerator(*args, **kw):
-                kw['acceptPort'] = port
-                kw['failure'] = connectionRefusedFailure
+            def tor_socks_endpoint_generator(*args, **kw):
+                kw['accept_port'] = port
+                kw['failure'] = Failure(ConnectionRefusedError())
                 return FakeTorSocksEndpoint(*args, **kw)
-            endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator)
+            endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=tor_socks_endpoint_generator)
             d = endpoint.connect(None)
             return self.assertFailure(d, ConnectionRefusedError)
 
@@ -658,13 +707,17 @@ class TestTorClientEndpoint(unittest.TestCase):
         This tests that if a SOCKS port is specified, we *only* attempt to
         connect to that SOCKS port.
         """
-        def TorSocksEndpointGenerator(*args, **kw):
-            kw['acceptPort'] = 6669
-            kw['failure'] = connectionRefusedFailure
-            return FakeTorSocksEndpoint(*args, **kw)
-        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator, socks_port=6669)
+        endpoints = []
+
+        def tor_socks_endpoint_generator(*args, **kw):
+            kw['accept_port'] = 6669
+            kw['failure'] = Failure(ConnectionRefusedError())
+            endpoints.append(FakeTorSocksEndpoint(*args, **kw))
+            return endpoints[-1]
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=tor_socks_endpoint_generator, socks_port=6669)
         endpoint.connect(None)
-        self.assertEqual(endpoint.tor_socks_endpoint.transport.value(), '\x05\x01\x00')
+        self.assertEqual(1, len(endpoints))
+        self.assertEqual(endpoints[-1].transport.value(), '\x05\x01\x00')
 
     def test_bad_no_guess_socks_port(self):
         """
@@ -672,10 +725,10 @@ class TestTorClientEndpoint(unittest.TestCase):
         specified SOCKS port... even if there is a valid SOCKS port listening on
         the socks_ports_to_try list.
         """
-        def TorSocksEndpointGenerator(*args, **kw):
-            kw['acceptPort'] = 9050
-            kw['failure'] = connectionRefusedFailure
+        def tor_socks_endpoint_generator(*args, **kw):
+            kw['accept_port'] = 9050
+            kw['failure'] = Failure(ConnectionRefusedError())
             return FakeTorSocksEndpoint(*args, **kw)
-        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator, socks_port=6669)
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=tor_socks_endpoint_generator, socks_port=6669)
         d = endpoint.connect(None)
         self.assertFailure(d, ConnectionRefusedError)
