@@ -166,6 +166,47 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(conf.foo is False)
         self.assertTrue(conf.bar is True)
 
+    def test_save_boolean(self):
+        self.protocol.answers.append('config/names=\nfoo Boolean\nbar Boolean')
+        self.protocol.answers.append({'foo': '0'})
+        self.protocol.answers.append({'bar': '1'})
+
+        conf = TorConfig(self.protocol)
+
+        # save some boolean value
+        conf.foo = True
+        conf.bar = False
+        conf.save()
+        self.assertEqual(set(self.protocol.sets),
+                         set([('foo', 1), ('bar', 0)]))
+
+    def test_read_boolean_after_save(self):
+        self.protocol.answers.append('config/names=\nfoo Boolean\nbar Boolean')
+        self.protocol.answers.append({'foo': '0'})
+        self.protocol.answers.append({'bar': '1'})
+
+        conf = TorConfig(self.protocol)
+
+        # save some boolean value
+        conf.foo = True
+        conf.bar = False
+        conf.save()
+        self.assertTrue(conf.foo is True, msg="foo not True: %s" % conf.foo)
+        self.assertTrue(conf.bar is False, msg="bar not False: %s" % conf.bar)
+
+    def test_save_boolean_with_strange_values(self):
+        self.protocol.answers.append('config/names=\nfoo Boolean\nbar Boolean')
+        self.protocol.answers.append({'foo': '0'})
+        self.protocol.answers.append({'bar': '1'})
+
+        conf = TorConfig(self.protocol)
+        # save some non-boolean value
+        conf.foo = "Something True"
+        conf.bar = 0
+        conf.save()
+        self.assertEqual(set(self.protocol.sets),
+                         set([('foo', 1), ('bar', 0)]))
+
     def test_boolean_auto_parser(self):
         self.protocol.answers.append(
             'config/names=\nfoo Boolean+Auto\nbar Boolean+Auto\nbaz Boolean+Auto'
@@ -179,6 +220,48 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(conf.bar is 1)
         self.assertTrue(conf.baz is -1)
 
+    def test_save_boolean_auto(self):
+        self.protocol.answers.append(
+            'config/names=\nfoo Boolean+Auto\nbar Boolean+Auto\nbaz Boolean+Auto\nqux Boolean+Auto'
+        )
+        self.protocol.answers.append({'foo': '1'})
+        self.protocol.answers.append({'bar': '1'})
+        self.protocol.answers.append({'baz': '1'})
+        self.protocol.answers.append({'qux': '1'})
+
+        conf = TorConfig(self.protocol)
+        conf.foo = 1
+        conf.bar = 0
+        conf.baz = True
+        conf.qux = -1
+        conf.save()
+        self.assertEqual(set(self.protocol.sets),
+                         set([('foo', 1),
+                              ('bar', 0),
+                              ('baz', 1),
+                              ('qux', 'auto')]))
+        self.assertTrue(conf.foo is 1)
+        self.assertTrue(conf.bar is 0)
+        self.assertTrue(conf.baz is 1)
+        self.assertTrue(conf.qux is -1)
+
+    def test_save_invalid_boolean_auto(self):
+        self.protocol.answers.append(
+            'config/names=\nfoo Boolean+Auto'
+        )
+        self.protocol.answers.append({'foo': '1'})
+
+        conf = TorConfig(self.protocol)
+        for value in ('auto', 'True', 'False', None):
+            try:
+                conf.foo = value
+            except (ValueError, TypeError):
+                pass
+            else:
+                self.fail("Invalid value '%s' allowed" % value)
+            conf.save()
+            self.assertEqual(self.protocol.sets, [])
+
     def test_string_parser(self):
         self.protocol.answers.append('config/names=\nfoo String')
         self.protocol.answers.append({'foo': 'bar'})
@@ -191,6 +274,39 @@ class ConfigTests(unittest.TestCase):
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, 123)
 
+    def test_int_validator(self):
+        self.protocol.answers.append('config/names=\nfoo Integer')
+        self.protocol.answers.append({'foo': '123'})
+        conf = TorConfig(self.protocol)
+
+        conf.foo = 2.33
+        conf.save()
+        self.assertEqual(conf.foo, 2)
+
+        conf.foo = '1'
+        conf.save()
+        self.assertEqual(conf.foo, 1)
+
+        conf.foo = '-100'
+        conf.save()
+        self.assertEqual(conf.foo, -100)
+
+        conf.foo = 0
+        conf.save()
+        self.assertEqual(conf.foo, 0)
+
+        conf.foo = '0'
+        conf.save()
+        self.assertEqual(conf.foo, 0)
+
+        for value in ('no', 'Not a value', None):
+            try:
+                conf.foo = value
+            except (ValueError, TypeError):
+                pass
+            else:
+                self.fail("No excpetion thrown")
+
     def test_int_parser_error(self):
         self.protocol.answers.append('config/names=\nfoo Integer')
         self.protocol.answers.append({'foo': '123foo'})
@@ -603,6 +719,41 @@ class EventTests(unittest.TestCase):
         self.assertEqual(config.Foo, 'bar')
         self.assertEqual(config.Bar, DEFAULT_VALUE)
 
+    def test_conf_changed_parsed(self):
+        '''
+        Create a configuration which holds boolean types. These types
+        have to be parsed as booleans.
+        '''
+        protocol = FakeControlProtocol([])
+        protocol.answers.append('config/names=\nFoo Boolean\nBar Boolean')
+        protocol.answers.append({'Foo': '0'})
+        protocol.answers.append({'Bar': '1'})
+
+        config = TorConfig(protocol)
+        # Initial value is not tested here
+        protocol.events['CONF_CHANGED']('Foo=1\nBar=0')
+
+        msg = "Foo is not True: %r" % config.Foo
+        self.assertTrue(config.Foo is True, msg=msg)
+
+        msg = "Foo is not False: %r" % config.Bar
+        self.assertTrue(config.Bar is False, msg=msg)
+
+    def test_conf_changed_invalid_values(self):
+        protocol = FakeControlProtocol([])
+        protocol.answers.append('config/names=\nFoo Integer\nBar Integer')
+        protocol.answers.append({'Foo': '0'})
+        protocol.answers.append({'Bar': '1'})
+
+        config = TorConfig(protocol)
+        # Initial value is not tested here
+        try:
+            protocol.events['CONF_CHANGED']('Foo=INVALID\nBar=VALUES')
+        except (ValueError, TypeError):
+            pass
+        else:
+            self.fail("No excpetion thrown")
+
 
 class CreateTorrcTests(unittest.TestCase):
 
@@ -709,6 +860,48 @@ HiddenServiceAuthorizeClient Dependant''')
         finally:
             shutil.rmtree(d, ignore_errors=True)
 
+    def test_single_client(self):
+        # FIXME test without crapping on filesystem
+        self.protocol.answers.append('HiddenServiceDir=/fake/path\n')
+        d = tempfile.mkdtemp()
+
+        try:
+            with open(os.path.join(d, 'hostname'), 'w') as f:
+                f.write('gobledegook\n')
+
+            conf = TorConfig(self.protocol)
+            hs = HiddenService(conf, d, [])
+
+            self.assertEqual(1, len(hs.clients))
+            self.assertEqual('default', hs.clients[0][0])
+            self.assertEqual('gobledegook', hs.clients[0][1])
+
+        finally:
+            shutil.rmtree(d, ignore_errors=True)
+
+    def test_stealth_clients(self):
+        # FIXME test without crapping on filesystem
+        self.protocol.answers.append('HiddenServiceDir=/fake/path\n')
+        d = tempfile.mkdtemp()
+
+        try:
+            with open(os.path.join(d, 'hostname'), 'w') as f:
+                f.write('oniona cookiea\n')
+                f.write('onionb cookieb\n')
+
+            conf = TorConfig(self.protocol)
+            hs = HiddenService(conf, d, [])
+
+            self.assertEqual(2, len(hs.clients))
+            self.assertEqual('oniona', hs.clients[0][0])
+            self.assertEqual('cookiea', hs.clients[0][1])
+            self.assertEqual('onionb', hs.clients[1][0])
+            self.assertEqual('cookieb', hs.clients[1][1])
+            self.assertRaises(RuntimeError, getattr, hs, 'hostname')
+
+        finally:
+            shutil.rmtree(d, ignore_errors=True)
+
     def test_modify_hidden_service(self):
         self.protocol.answers.append('HiddenServiceDir=/fake/path\nHiddenServicePort=80 127.0.0.1:1234\n')
 
@@ -1545,11 +1738,11 @@ class HiddenServiceAuthTests(unittest.TestCase):
         self.assertEqual(3, len(clients))
         self.assertEqual('bar', clients[0].name)
         self.assertEqual('O4rQyZ+IJr2PNHUdeXi0nA', clients[0].cookie)
-        self.assertEqual('MIICXQIBAAKBgQC1R/bPGTWnpGJpNCfT1KIfFq1QEGHz4enKSEKUDkz1CSEPOMGSbV37dfqTuI4klsFvdUsR3NpYXLin9xRWvw1viKwAN0y8cv5totl4qMxO5i+zcfVhbJiNvVv2EjfEyQaZfAy2PUfp/tAPYZMsyfps2DptWyNR', clients[0].key)
+        self.assertEqual('RSA1024:MIICXQIBAAKBgQC1R/bPGTWnpGJpNCfT1KIfFq1QEGHz4enKSEKUDkz1CSEPOMGSbV37dfqTuI4klsFvdUsR3NpYXLin9xRWvw1viKwAN0y8cv5totl4qMxO5i+zcfVhbJiNvVv2EjfEyQaZfAy2PUfp/tAPYZMsyfps2DptWyNR', clients[0].key)
 
         self.assertEqual('foo', clients[1].name)
         self.assertEqual('btlj4+RsWEkxigmlszInhQ', clients[1].cookie)
-        self.assertEqual(clients[1].key, 'MIICXgIBAAKBgQDdLdHU1fbABtFutOFtpdWQdv/9qG1OAc0r1TfaBtkPSNcLezcxSThalIEnRFfejy0suOHmsqspruvn0FEflIEQvFWeXAPvXg==')
+        self.assertEqual(clients[1].key, 'RSA1024:MIICXgIBAAKBgQDdLdHU1fbABtFutOFtpdWQdv/9qG1OAc0r1TfaBtkPSNcLezcxSThalIEnRFfejy0suOHmsqspruvn0FEflIEQvFWeXAPvXg==')
 
         self.assertEqual('quux', clients[2].name)
         self.assertEqual('asdlkjasdlfkjalsdkfffj', clients[2].cookie)
@@ -1562,3 +1755,117 @@ class HiddenServiceAuthTests(unittest.TestCase):
             RuntimeError,
             parse_client_keys, data
         )
+
+
+class EphemeralHiddenServiceTest(unittest.TestCase):
+    def test_defaults(self):
+        eph = torconfig.EphemeralHiddenService("80 localhost:80")
+        self.assertEqual(eph._ports, ["80,localhost:80"])
+
+    def test_wrong_blob(self):
+        try:
+            eph = torconfig.EphemeralHiddenService("80 localhost:80", "foo")
+            self.fail("should get exception")
+        except RuntimeError as e:
+            pass
+
+    def test_add(self):
+        eph = torconfig.EphemeralHiddenService("80 127.0.0.1:80")
+        proto = Mock()
+        proto.queue_command = Mock(return_value="PrivateKey=blam\nServiceID=ohai")
+        eph.add_to_tor(proto)
+
+        self.assertEqual("blam", eph.private_key)
+        self.assertEqual("ohai.onion", eph.hostname)
+
+    def test_descriptor_wait(self):
+        eph = torconfig.EphemeralHiddenService("80 127.0.0.1:80")
+        proto = Mock()
+        proto.queue_command = Mock(return_value=defer.succeed("PrivateKey=blam\nServiceID=ohai\n"))
+
+        eph.add_to_tor(proto)
+
+        # get the event-listener callback that torconfig code added;
+        # the last call [-1] was to add_event_listener; we want the
+        # [1] arg of that
+        cb = proto.method_calls[-1][1][1]
+
+        # Tor doesn't actually provide the .onion, but we can test it anyway
+        cb('UPLOADED ohai UNKNOWN somehsdir')
+        cb('UPLOADED UNKNOWN UNKNOWN somehsdir')
+
+        self.assertEqual("blam", eph.private_key)
+        self.assertEqual("ohai.onion", eph.hostname)
+
+    def test_remove(self):
+        eph = torconfig.EphemeralHiddenService("80 127.0.0.1:80")
+        eph.hostname = 'foo.onion'
+        proto = Mock()
+        proto.queue_command = Mock(return_value="OK")
+
+        eph.remove_from_tor(proto)
+
+    @defer.inlineCallbacks
+    def test_remove_error(self):
+        eph = torconfig.EphemeralHiddenService("80 127.0.0.1:80")
+        eph.hostname = 'foo.onion'
+        proto = Mock()
+        proto.queue_command = Mock(return_value="it's not ok")
+
+        try:
+            yield eph.remove_from_tor(proto)
+            self.fail("should have gotten exception")
+        except RuntimeError as e:
+            pass
+
+    def test_failed_upload(self):
+        eph = torconfig.EphemeralHiddenService("80 127.0.0.1:80")
+        proto = Mock()
+        proto.queue_command = Mock(return_value=defer.succeed("PrivateKey=seekrit\nServiceID=42\n"))
+
+        d = eph.add_to_tor(proto)
+
+        # get the event-listener callback that torconfig code added;
+        # the last call [-1] was to add_event_listener; we want the
+        # [1] arg of that
+        cb = proto.method_calls[-1][1][1]
+
+        # Tor leads with UPLOAD events for each attempt; we queue 2 of
+        # these...
+        cb('UPLOAD 42 UNKNOWN hsdir0')
+        cb('UPLOAD 42 UNKNOWN hsdir1')
+
+        # ...but fail them both
+        cb('FAILED 42 UNKNOWN hsdir1 REASON=UPLOAD_REJECTED')
+        cb('FAILED 42 UNKNOWN hsdir0 REASON=UPLOAD_REJECTED')
+
+        self.assertEqual("seekrit", eph.private_key)
+        self.assertEqual("42.onion", eph.hostname)
+        self.assertTrue(d.called)
+        d.addErrback(lambda e: self.assertTrue('Failed to upload' in str(e)))
+
+    def test_single_failed_upload(self):
+        eph = torconfig.EphemeralHiddenService("80 127.0.0.1:80")
+        proto = Mock()
+        proto.queue_command = Mock(return_value=defer.succeed("PrivateKey=seekrit\nServiceID=42\n"))
+
+        d = eph.add_to_tor(proto)
+
+        # get the event-listener callback that torconfig code added;
+        # the last call [-1] was to add_event_listener; we want the
+        # [1] arg of that
+        cb = proto.method_calls[-1][1][1]
+
+        # Tor leads with UPLOAD events for each attempt; we queue 2 of
+        # these...
+        cb('UPLOAD 42 UNKNOWN hsdir0')
+        cb('UPLOAD 42 UNKNOWN hsdir1')
+
+        # ...then fail one
+        cb('FAILED 42 UNKNOWN hsdir1 REASON=UPLOAD_REJECTED')
+        # ...and succeed on the last.
+        cb('UPLOADED 42 UNKNOWN hsdir0')
+
+        self.assertEqual("seekrit", eph.private_key)
+        self.assertEqual("42.onion", eph.hostname)
+        self.assertTrue(d.called)
