@@ -213,7 +213,7 @@ class TCPHiddenServiceEndpoint(object):
 
     @classmethod
     def global_tor(cls, reactor, public_port, hidden_service_dir=None,
-                   local_port=None, control_port=None):
+                   local_port=None, control_port=None, stealth_auth=None):
         """
         This returns a TCPHiddenServiceEndpoint connected to a
         txtorcon global Tor instance. The first time you call this, a
@@ -232,17 +232,27 @@ class TCPHiddenServiceEndpoint(object):
 
         All keyword options have defaults (e.g. random ports, or
         tempdirs).
+
+        :param stealth_auth:
+            None, or a list of strings -- one for each stealth
+            authenticator you require.
         """
 
         def progress(*args):
             progress.target(*args)
-        config = get_global_tor(reactor,
-                                control_port=control_port,
-                                progress_updates=progress)
-        # config is a Deferred here, but endpoint resolves in listen()
-        r = TCPHiddenServiceEndpoint(reactor, config, public_port,
-                                     hidden_service_dir=hidden_service_dir,
-                                     local_port=local_port)
+        config = get_global_tor(
+            reactor,
+            control_port=control_port,
+            progress_updates=progress
+        )
+        # config is a Deferred here, but endpoint resolves it in
+        # the listen() call
+        r = TCPHiddenServiceEndpoint(
+            reactor, config, public_port,
+            hidden_service_dir=hidden_service_dir,
+            local_port=local_port,
+            stealth_auth=stealth_auth,
+        )
         progress.target = r._tor_progress_update
         return r
 
@@ -274,7 +284,8 @@ class TCPHiddenServiceEndpoint(object):
         return r
 
     def __init__(self, reactor, config, public_port,
-                 hidden_service_dir=None, local_port=None):
+                 hidden_service_dir=None, local_port=None,
+                 stealth_auth=None):
         """
         :param reactor:
             :api:`twisted.internet.interfaces.IReactorTCP` provider
@@ -300,6 +311,10 @@ class TCPHiddenServiceEndpoint(object):
             not provided, one is created with temp.mkstemp() AND
             DELETED when the reactor shuts down.
 
+        :param stealth_auth:
+            A list of strings, one name for each stealth authenticator
+            you want. Like: ``['alice', 'bob']``
+
         :param endpoint_generator:
             A callable that generates a new instance of something that
             implements IServerEndpoint (by default TCP4ServerEndpoint)
@@ -309,6 +324,7 @@ class TCPHiddenServiceEndpoint(object):
         self.config = defer.maybeDeferred(lambda: config)
         self.public_port = public_port
         self.local_port = local_port
+        self.stealth_auth = stealth_auth
 
         self.hidden_service_dir = hidden_service_dir
         self.tcp_listening_port = None
@@ -421,11 +437,17 @@ class TCPHiddenServiceEndpoint(object):
                 info_callback.callback(None)
         self.config.protocol.add_event_listener('INFO', info_event)
 
-        if self.hidden_service_dir not in [hs.dir for hs in self.config.HiddenServices]:
+        hs_dirs = [hs.dir for hs in self.config.HiddenServices]
+        if self.hidden_service_dir not in hs_dirs:
+            authlines = []
+            if self.stealth_auth:
+                # like "stealth name0,name1"
+                authlines = ['stealth ' + ','.join(self.stealth_auth)]
             self.hiddenservice = HiddenService(
                 self.config, self.hidden_service_dir,
                 ['%d 127.0.0.1:%d' % (self.public_port, self.local_port)],
-                group_readable=1)
+                group_readable=1, auth=authlines,
+            )
             self.config.HiddenServices.append(self.hiddenservice)
         yield self.config.save()
 
@@ -436,15 +458,29 @@ class TCPHiddenServiceEndpoint(object):
         self._tor_progress_update(100.0, 'wait_descriptor',
                                   'At least one descriptor uploaded.')
 
-        log.msg(
-            'Started hidden service "%s" on port %d' %
-            (self.onion_uri, self.public_port))
-        log.msg('Keys are in "%s".' % (self.hidden_service_dir,))
-        defer.returnValue(TorOnionListeningPort(self.tcp_listening_port,
-                                                self.hidden_service_dir,
-                                                self.onion_uri,
-                                                self.public_port,
-                                                self.config))
+        # FIXME XXX need to work out what happens here on stealth-auth'd
+        # things. maybe we need a separate StealthHiddenService
+        # vs. HiddenService ?!
+        # XXX that is, self.onion_uri isn't always avaialble :/
+
+        uri = None
+        if self.hiddenservice is not None:
+            log.msg('Started hidden service port %d' % self.public_port)
+            for client in self.hiddenservice.clients:
+                # XXX FIXME just taking the first one on multi-client services
+                if uri is None:
+                    uri = client[1]
+                log.msg('  listening on %s.onion' % client[1])
+
+        defer.returnValue(
+            TorOnionListeningPort(
+                self.tcp_listening_port,
+                self.hidden_service_dir,
+                uri,
+                self.public_port,
+                self.config,
+            )
+        )
 
 
 @implementer(IAddress)
@@ -638,10 +674,10 @@ class TorClientEndpoint(object):
             raise ValueError('host and port must be specified')
 
         self.host = host
-        self.port = port
+        self.port = int(port)
         self._proxy_endpoint_generator = _proxy_endpoint_generator
         self.socks_hostname = socks_hostname
-        self.socks_port = socks_port
+        self.socks_port = int(socks_port) if socks_port is not None else None
         self.socks_username = socks_username
         self.socks_password = socks_password
 
@@ -649,52 +685,37 @@ class TorClientEndpoint(object):
             self._socks_port_iter = iter(self.socks_ports_to_try)
             self._socks_guessing_enabled = True
         else:
+            self._socks_port_iter = [socks_port]
             self._socks_guessing_enabled = False
 
+    @defer.inlineCallbacks
     def connect(self, protocolfactory):
-        self.protocolfactory = protocolfactory
-
-        if self._socks_guessing_enabled:
-            self.socks_port = self._socks_port_iter.next()
-
-        d = self._try_connect()
-        return d
-
-    def _try_connect(self):
-        self.tor_socks_endpoint = self._proxy_endpoint_generator(
-            reactor,
-            self.socks_hostname,
-            self.socks_port
-        )
-
-        if self.socks_username is None or self.socks_password is None:
-            ep = SOCKS5ClientEndpoint(
-                self.host,
-                self.port,
-                self.tor_socks_endpoint
-            )
-        else:
-            ep = SOCKS5ClientEndpoint(
-                self.host,
-                self.port,
-                self.tor_socks_endpoint,
-                methods=dict(login=(self.socks_username, self.socks_password))
+        last_error = None
+        for socks_port in self._socks_port_iter:
+            self.socks_port = socks_port
+            tor_ep = self._proxy_endpoint_generator(
+                reactor,
+                self.socks_hostname,
+                self.socks_port,
             )
 
-        d = ep.connect(self.protocolfactory)
-        if self._socks_guessing_enabled:
-            d.addErrback(self._retry_socks_port)
-        return d
+            args = (self.host, self.port, tor_ep)
+            kwargs = dict()
+            if self.socks_username is not None and self.socks_password is not None:
+                kwargs['methods'] = dict(
+                    login=(self.socks_username, self.socks_password),
+                )
 
-    def _retry_socks_port(self, failure):
-        failure.trap(error.ConnectError)
-        try:
-            self.socks_port = self._socks_port_iter.next()
-        except StopIteration:
-            return failure
-        d = self._try_connect()
-        d.addErrback(self._retry_socks_port)
-        return d
+            socks_ep = SOCKS5ClientEndpoint(*args, **kwargs)
+
+            try:
+                proto = yield socks_ep.connect(protocolfactory)
+                defer.returnValue(proto)
+
+            except error.ConnectError as e0:
+                last_error = e0
+        if last_error is not None:
+            raise last_error
 
 
 @implementer(IPlugin, IStreamClientEndpointStringParserWithReactor)
