@@ -6,24 +6,24 @@ from __future__ import with_statement
 
 import os
 import sys
+import types
 import functools
 import tempfile
 import warnings
 from io import StringIO
 import shlex
-if sys.platform in ('linux2', 'darwin'):
-    import pwd
 
 from twisted.python import log
-from twisted.python.failure import Failure
 from twisted.internet import defer, error, protocol
 from twisted.internet.interfaces import IReactorTime
 from twisted.internet.endpoints import TCP4ClientEndpoint
 
-from txtorcon.torcontrolprotocol import parse_keywords, TorProtocolFactory
+from txtorcon.torcontrolprotocol import parse_keywords, TorProtocolFactory, DEFAULT_VALUE
 from txtorcon.util import delete_file_or_tree, find_keywords, find_tor_binary
 from txtorcon.log import txtorlog
 from txtorcon.interface import ITorControlProtocol
+if sys.platform in ('linux2', 'darwin'):
+    import pwd
 
 
 class TorNotFound(RuntimeError):
@@ -111,7 +111,6 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         self.kill_on_stderr = kill_on_stderr
         self.stderr = stderr
         self.stdout = stdout
-        self.collected_stdout = StringIO()
 
         self._setup_complete = False
         self._did_timeout = False
@@ -195,7 +194,8 @@ class TorProcessProtocol(protocol.ProcessProtocol):
                     "Tor was killed (%s)." % status.value.signal)
         else:
             err = RuntimeError(
-                "Tor exited with error-code %d" % status.value.exitCode)
+                "Tor exited with error-code %d" % status.value.exitCode
+            )
 
         log.err(err)
         if self.connected_cb:
@@ -465,7 +465,7 @@ def launch_tor(config, reactor,
             tor_binary,
             args=args,
             env={'HOME': data_directory},
-            path=data_directory
+            path=data_directory if os.path.exists(data_directory) else None,
         )
         # FIXME? don't need rest of the args: uid, gid, usePTY, childFDs)
         transport.closeStdin()
@@ -500,11 +500,17 @@ class TorConfigType(object):
 
 
 class Boolean(TorConfigType):
+    "Boolean values are stored as 0 or 1."
     def parse(self, s):
         if int(s):
             return True
         return False
 
+    def validate(self, s, instance, name):
+        if s:
+            return 1
+        return 0
+
 
 class Boolean_Auto(TorConfigType):
     """
@@ -520,11 +526,24 @@ class Boolean_Auto(TorConfigType):
             return 1
         return 0
 
+    def validate(self, s, instance, name):
+        # FIXME: Is 'auto' an allowed value? (currently not)
+        s = int(s)
+        if s < 0:
+            return 'auto'
+        elif s:
+            return 1
+        else:
+            return 0
+
 
 class Integer(TorConfigType):
     def parse(self, s):
         return int(s)
 
+    def validate(self, s, instance, name):
+        return int(s)
+
 
 class SignedInteger(Integer):
     pass
@@ -741,9 +760,52 @@ class HiddenService(object):
         self.__dict__[name] = value
 
     def __getattr__(self, name):
-        if name in ('hostname', 'private_key'):
+        '''
+        FIXME can't we just move this to @property decorated methods
+        instead?
+        '''
+
+        # For stealth authentication, the .onion is per-client. So in
+        # that case, we really have no choice here -- we can't have
+        # "a" hostname. So we just barf; it's an error to access to
+        # hostname this way. Instead, use .clients.{hostname, cookie}
+
+        if name == 'private_key':
             with open(os.path.join(self.dir, name)) as f:
-                self.__dict__[name] = f.read().strip()
+                data = f.read().strip()
+            self.__dict__[name] = data
+
+        elif name == 'clients':
+            clients = []
+            try:
+                with open(os.path.join(self.dir, 'hostname')) as f:
+                    for line in f.readlines():
+                        args = line.split()
+                        # XXX should be a dict?
+                        if len(args) > 1:
+                            # tag, onion-uri?
+                            clients.append((args[0], args[1]))
+                        else:
+                            clients.append(('default', args[0]))
+            except IOError:
+                pass
+            self.__dict__[name] = clients
+
+        elif name == 'hostname':
+            with open(os.path.join(self.dir, name)) as f:
+                data = f.read().strip()
+            host = None
+            for line in data.split('\n'):
+                h = line.split(' ')[0]
+                if host is None:
+                    host = h
+                elif h != host:
+                    raise RuntimeError(
+                        ".hostname accessed on stealth-auth'd hidden-service "
+                        "with multiple onion addresses."
+                    )
+            self.__dict__[name] = h
+
         elif name == 'client_keys':
             fname = os.path.join(self.dir, name)
             keys = []
@@ -762,8 +824,8 @@ class HiddenService(object):
         if self.conf._supports['HiddenServiceDirGroupReadable'] \
            and self.group_readable:
             rtn.append(('HiddenServiceDirGroupReadable', str(1)))
-        for x in self.ports:
-            rtn.append(('HiddenServicePort', str(x)))
+        for port in self.ports:
+            rtn.append(('HiddenServicePort', str(port)))
         if self.version:
             rtn.append(('HiddenServiceVersion', str(self.version)))
         for authline in self.authorize_client:
@@ -771,8 +833,121 @@ class HiddenService(object):
         return rtn
 
 
+class EphemeralHiddenService(object):
+    '''
+    This uses the ephemeral hidden-service APIs (in comparison to
+    torrc or SETCONF). This means your hidden-service private-key is
+    never in a file. It also means that when the process exits, that
+    HS goes away. See documentation for ADD_ONION in torspec:
+    https://gitweb.torproject.org/torspec.git/tree/control-spec.txt#n1295
+    '''
+
+    # XXX the "ports" stuff is still kind of an awkward API, especialy
+    # making the actual list public (since it'll have
+    # "80,127.0.0.1:80" instead of with a space
+
+    # XXX descriptor upload stuff needs more features from Tor (the
+    # actual uploaded key; the event always says UNKNOWN)
+
+    # XXX "auth" is unused (also, no Tor support I don't think?)
+
+    def __init__(self, ports, key_blob_or_type='NEW:BEST', auth=[], ver=2):
+        if not isinstance(ports, types.ListType):
+            ports = [ports]
+        # for "normal" HSes the port-config bit looks like "80
+        # 127.0.0.1:1234" whereas this one wants a comma, so we leave
+        # the public API the same and fix up the space. Or of course
+        # you can just use the "real" comma-syntax if you wanted.
+        self._ports = map(lambda x: x.replace(' ', ','), ports)
+        self._key_blob = key_blob_or_type
+        self.auth = auth  # FIXME ununsed
+        # FIXME nicer than assert, plz
+        assert ' ' not in self._key_blob
+        assert isinstance(ports, types.ListType)
+        if not key_blob_or_type.startswith('NEW:') \
+           and (len(key_blob_or_type) > 825 or len(key_blob_or_type) < 820):
+            raise RuntimeError('Wrong size key-blob')
+
+    @defer.inlineCallbacks
+    def add_to_tor(self, protocol):
+        '''
+        Returns a Deferred which fires with 'self' after at least one
+        descriptor has been uploaded. Errback if no descriptor upload
+        succeeds.
+        '''
+        ports = ' '.join(map(lambda x: 'Port=' + x.strip(), self._ports))
+        cmd = 'ADD_ONION %s %s' % (self._key_blob, ports)
+        ans = yield protocol.queue_command(cmd)
+        ans = find_keywords(ans.split('\n'))
+        self.hostname = ans['ServiceID'] + '.onion'
+        if self._key_blob == 'NEW:BEST':
+            self.private_key = ans['PrivateKey']
+
+        log.msg('Created hidden-service at', self.hostname)
+
+        # Now we want to wait for the descriptor uploads. This doesn't
+        # quite work, as the UPLOADED events always say "UNKNOWN" for
+        # the HSAddress so we can't correlate it to *this* onion for
+        # sure :/ "yet", though. Yawning says on IRC this is coming.
+
+        # XXX Hmm, still UPLOADED always says UNKNOWN, but the UPLOAD
+        # events do say the address -- so we save all those, and
+        # correlate to the target nodes. Not sure if this will really
+        # even work, but better than nothing.
+
+        uploaded = defer.Deferred()
+        attempted_uploads = set()
+        confirmed_uploads = set()
+        failed_uploads = set()
+
+        def hs_desc(evt):
+            """
+            From control-spec:
+            "650" SP "HS_DESC" SP Action SP HSAddress SP AuthType SP HsDir
+            [SP DescriptorID] [SP "REASON=" Reason] [SP "REPLICA=" Replica]
+            """
+
+            args = evt.split()
+            subtype = args[0]
+            if subtype == 'UPLOAD':
+                if args[1] == self.hostname[:-6]:
+                    attempted_uploads.add(args[3])
+
+            elif subtype == 'UPLOADED':
+                # we only need ONE successful upload to happen for the
+                # HS to be reachable. (addr is args[1])
+                if args[3] in attempted_uploads:
+                    confirmed_uploads.add(args[3])
+                    log.msg("Uploaded '{}' to '{}'".format(self.hostname, args[3]))
+                    uploaded.callback(self)
+
+            elif subtype == 'FAILED':
+                if args[1] == self.hostname[:-6]:
+                    failed_uploads.add(args[3])
+                    if failed_uploads == attempted_uploads:
+                        msg = "Failed to upload '{}' to: {}".format(
+                            self.hostname,
+                            ', '.join(failed_uploads),
+                        )
+                        uploaded.errback(RuntimeError(msg))
+
+        log.msg("Created '{}', waiting for descriptor uploads.".format(self.hostname))
+        yield protocol.add_event_listener('HS_DESC', hs_desc)
+        yield uploaded
+        yield protocol.remove_event_listener('HS_DESC', hs_desc)
+
+    @defer.inlineCallbacks
+    def remove_from_tor(self, protocol):
+        '''
+        Returns a Deferred which fires with None
+        '''
+        r = yield protocol.queue_command('DEL_ONION %s' % self.hostname[:-6])
+        if r.strip() != 'OK':
+            raise RuntimeError('Failed to remove hidden service: "%s".' % r)
+
+
 def parse_rsa_blob(lines):
-    return ''.join(lines[1:-1])
+    return 'RSA1024:' + ''.join(lines[1:-1])
 
 
 def parse_client_keys(stream):
@@ -908,6 +1083,18 @@ class TorConfig(object):
 
     """
 
+    @classmethod
+    @defer.inlineCallbacks
+    def from_protocol(cls, proto):
+        """
+        This creates and returns a ready-to-go TorConfig instance from the
+        given protocol, which should be an instance of
+        TorControlProtocol.
+        """
+        cfg = TorConfig(control=proto)
+        yield cfg.post_bootstrap
+        defer.returnValue(cfg)
+
     def __init__(self, control=None):
         self.config = {}
         '''Current configuration, by keys.'''
@@ -1081,7 +1268,10 @@ class TorConfig(object):
         for (k, v) in conf.items():
             # v will be txtorcon.DEFAULT_VALUE already from
             # parse_keywords if it was unspecified
-            self.config[self._find_real_name(k)] = v
+            real_name = self._find_real_name(k)
+            if real_name in self.parsers:
+                v = self.parsers[real_name].parse(v)
+            self.config[real_name] = v
 
     def bootstrap(self, arg=None):
         '''
@@ -1150,8 +1340,10 @@ class TorConfig(object):
 
             if isinstance(value, list):
                 for x in value:
-                    args.append(key)
-                    args.append(str(x))
+                    # FIXME XXX
+                    if x is not DEFAULT_VALUE:
+                        args.append(key)
+                        args.append(str(x))
 
             else:
                 args.append(key)
@@ -1159,7 +1351,10 @@ class TorConfig(object):
 
             # FIXME in future we should wait for CONF_CHANGED and
             # update then, right?
-            self.config[self._find_real_name(key)] = value
+            real_name = self._find_real_name(key)
+            if not isinstance(value, list) and real_name in self.parsers:
+                value = self.parsers[real_name].parse(value)
+            self.config[real_name] = value
 
         # FIXME might want to re-think this, but currently there's no
         # way to put things into a config and get them out again
