@@ -12,7 +12,7 @@ from twisted.protocols.basic import LineOnlyReceiver
 
 from zope.interface import implementer
 
-from txtorcon.util import hmac_sha256, compare_via_hash
+from txtorcon.util import hmac_sha256, compare_via_hash, unescape_quoted_string
 from txtorcon.log import txtorlog
 
 from txtorcon.interface import ITorControlProtocol
@@ -25,6 +25,39 @@ import base64
 DEFAULT_VALUE = 'DEFAULT'
 
 
+@defer.inlineCallbacks
+def connect(endpoint, password_function=lambda: None):
+    """
+    (experimental; details may change) This connects to the given
+    endpoint, presuming it is a Tor control connection and gives back
+    a TorControlProtocol instance. This does *not* reconnect if the
+    connection is dropped. This is the preferred entry point to
+    control a running Tor; if you need to *start* a Tor see
+    :ref:`launching_tor`.
+
+    See :meth:`txtorcon.TorState.from_protocol` to create a valid
+    :class:`txtorcon.TorState` instance from the connection.
+
+    :param endpoint: A Twisted IEndpoint, in practice either a
+        TCP4ClientEndpoint or a UnixClientEndpoint. By default, Tor
+        uses "tcp:localhost:9051" or "unix:/var/run/tor/control".
+
+    :param password_function: This is only consulted if the Tor to
+        which we connect does not have cookie authentication enabled. In
+        that case, this function is called to request a password. It may
+        return a Deferred.
+
+    :returns: A Deferred that fires with a ready-to-use
+        TorControlProtocol instance is returned, or Failure if a
+        connection can't be established.
+    """
+
+    factory = TorProtocolFactory(password_function=password_function)
+    proto = yield endpoint.connect(factory)
+    yield proto.post_bootstrap
+    defer.returnValue(proto)
+
+
 class TorProtocolError(RuntimeError):
     """
     Happens on 500-level responses in the protocol, almost certainly
@@ -58,8 +91,9 @@ class TorProtocolFactory(object):
         Builds protocols to talk to a Tor client on the specified
         address. For example::
 
-        TCP4ClientEndpoint(reactor, "localhost", 9051).connect(TorProtocolFactory())
-        reactor.run()
+            ep = TCP4ClientEndpoint(reactor, "localhost", 9051)
+            ep.connect(TorProtocolFactory())
+            reactor.run()
 
         By default, COOKIE authentication is used if
         available.
@@ -120,7 +154,7 @@ def unquote(word):
     return word
 
 
-def parse_keywords(lines, multiline_values=True):
+def parse_keywords(lines, multiline_values=True, key_hints=None):
     """
     Utility method to parse name=value pairs (GETINFO etc). Takes a
     string with newline-separated lines and expects at most one = sign
@@ -142,7 +176,11 @@ def parse_keywords(lines, multiline_values=True):
         if line.strip() == 'OK':
             continue
 
-        if '=' in line and ' ' not in line.split('=', 1)[0]:
+        sp = line.split('=', 1)
+        found_key = ('=' in line and ' ' not in sp[0])
+        if found_key and key_hints and sp[0] not in key_hints:
+            found_key = False
+        if found_key:
             if key:
                 if key in rtn:
                     if isinstance(rtn[key], list):
@@ -188,7 +226,7 @@ class TorControlProtocol(LineOnlyReceiver):
     :meth:`txtorcon.TorState.build_circuit` allows you to build custom
     circuits.
 
-   :meth:`txtorcon.TorControlProtocol.add_event_listener` can be used
+    :meth:`txtorcon.TorControlProtocol.add_event_listener` can be used
     to listen for specific events.
 
     To see how circuit and stream listeners are used, see
@@ -209,6 +247,9 @@ class TorControlProtocol(LineOnlyReceiver):
         authentication to Tor (default is to use COOKIE, however). May
         return Deferred."""
 
+        self._cookie_data = None
+        """Data read from cookie file used to authenticate."""
+
         self.version = None
         """Version of Tor we've connected to."""
 
@@ -244,7 +285,8 @@ class TorControlProtocol(LineOnlyReceiver):
             def setup(proto):
                 proto.post_bootstrap.addCallback(setup_complete)
 
-            TCP4ClientEndpoint(reactor, "localhost", 9051).connect(TorProtocolFactory())
+            ep = TCP4ClientEndpoint(reactor, "localhost", 9051)
+            ep.connect(TorProtocolFactory())
             d.addCallback(setup)
 
         See the helper method :func:`txtorcon.build_tor_connection`.
@@ -344,7 +386,6 @@ class TorControlProtocol(LineOnlyReceiver):
     # The following methods are the main TorController API and
     # probably the most interesting for users.
 
-    @defer.inlineCallbacks
     def get_info(self, *args):
         """
         Uses GETINFO to obtain informatoin from Tor.
@@ -362,16 +403,9 @@ class TorControlProtocol(LineOnlyReceiver):
             the keys you asked for. If you want to avoid the parsing
             into a dict, you can use get_info_raw instead.
         """
-        lines = yield self.get_info_raw(*args)
-        rtn = {}
-        key = None
-        for line in lines.split('\n'):
-            if line.split('=', 1)[0] in args:
-                key = line.split('=', 1)[0]
-                rtn[key] = line.split('=', 1)[1]
-            else:
-                rtn[key] = rtn[key] + '\n' + line
-        defer.returnValue(rtn)
+        d = self.get_info_raw(*args)
+        d.addCallback(parse_keywords, key_hints=args)
+        return d
 
     def get_conf(self, *args):
         """
@@ -618,7 +652,8 @@ class TorControlProtocol(LineOnlyReceiver):
         """
         Callback on AUTHCHALLENGE SAFECOOKIE
         """
-
+        if self._cookie_data is None:
+            raise RuntimeError("Cookie data not read.")
         kw = parse_keywords(reply.replace(' ', '\n'))
 
         server_hash = base64.b16decode(kw['SERVERHASH'])
@@ -626,7 +661,7 @@ class TorControlProtocol(LineOnlyReceiver):
         # FIXME put string in global. or something.
         expected_server_hash = hmac_sha256(
             "Tor safe cookie authentication server-to-controller hash",
-            self.cookie_data + self.client_nonce + server_nonce
+            self._cookie_data + self.client_nonce + server_nonce
         )
 
         if not compare_via_hash(expected_server_hash, server_hash):
@@ -638,18 +673,31 @@ class TorControlProtocol(LineOnlyReceiver):
 
         client_hash = hmac_sha256(
             "Tor safe cookie authentication controller-to-server hash",
-            self.cookie_data + self.client_nonce + server_nonce
+            self._cookie_data + self.client_nonce + server_nonce
         )
         client_hash_hex = base64.b16encode(client_hash)
         return self.queue_command('AUTHENTICATE %s' % client_hash_hex)
 
+    def _read_cookie(self, cookiefile):
+        """
+        Open and read a cookie file
+        :param cookie: Path to the cookie file
+        """
+        self._cookie_data = None
+        self._cookie_data = open(cookiefile, 'rb').read()
+        if len(self._cookie_data) != 32:
+            raise RuntimeError(
+                "Expected authentication cookie to be 32 bytes, got %d" %
+                len(self._cookie_data)
+            )
+
     def _do_authenticate(self, protoinfo):
         """
         Callback on PROTOCOLINFO to actually authenticate once we know
         what's supported.
         """
-
         methods = None
+        cookie_auth = False
         for line in protoinfo.split('\n'):
             if line[:5] == 'AUTH ':
                 kw = parse_keywords(line[5:].replace(' ', '\n'))
@@ -659,48 +707,56 @@ class TorControlProtocol(LineOnlyReceiver):
                 "Didn't find AUTH line in PROTOCOLINFO response."
             )
 
-        if 'SAFECOOKIE' in methods:
-            cookie = re.search('COOKIEFILE="(.*)"', protoinfo).group(1)
-            self.cookie_data = open(cookie, 'r').read()
-            if len(self.cookie_data) != 32:
-                raise RuntimeError(
-                    "Expected authentication cookie to be 32 bytes, got %d" %
-                    len(self.cookie_data)
-                )
-            txtorlog.msg("Using SAFECOOKIE authentication", cookie,
-                         len(self.cookie_data), "bytes")
-            self.client_nonce = os.urandom(32)
+        if 'SAFECOOKIE' in methods or 'COOKIE' in methods:
+            cookiefile_match = re.search(r'COOKIEFILE=("(?:[^"\\]|\\.)*")',
+                                         protoinfo)
+            if cookiefile_match:
+                cookiefile = cookiefile_match.group(1)
+                cookiefile = unescape_quoted_string(cookiefile)
+                try:
+                    self._read_cookie(cookiefile)
+                except IOError as why:
+                    txtorlog.msg("Reading COOKIEFILE failed: " + str(why))
+                    cookie_auth = False
+                else:
+                    cookie_auth = True
+            else:
+                txtorlog.msg("Didn't get COOKIEFILE")
 
-            cmd = 'AUTHCHALLENGE SAFECOOKIE ' + \
-                  base64.b16encode(self.client_nonce)
-            d = self.queue_command(cmd)
-            d.addCallback(self._safecookie_authchallenge)
-            d.addCallback(self._bootstrap)
-            d.addErrback(self._auth_failed)
-            return
+        if cookie_auth:
+            if 'SAFECOOKIE' in methods:
+                txtorlog.msg("Using SAFECOOKIE authentication", cookiefile,
+                             len(self._cookie_data), "bytes")
+                self.client_nonce = os.urandom(32)
 
-        elif 'COOKIE' in methods:
-            cookie = re.search('COOKIEFILE="(.*)"', protoinfo).group(1)
-            with open(cookie, 'r') as cookiefile:
-                data = cookiefile.read()
-            if len(data) != 32:
-                raise RuntimeError(
-                    "Expected authentication cookie to be 32 "
-                    "bytes, got %d instead." % len(data)
-                )
-            txtorlog.msg("Using COOKIE authentication",
-                         cookie, len(data), "bytes")
-            d = self.authenticate(data)
-            d.addCallback(self._bootstrap)
-            d.addErrback(self._auth_failed)
-            return
+                cmd = 'AUTHCHALLENGE SAFECOOKIE ' + \
+                      base64.b16encode(self.client_nonce)
+                d = self.queue_command(cmd)
+                d.addCallback(self._safecookie_authchallenge)
+                d.addCallback(self._bootstrap)
+                d.addErrback(self._auth_failed)
+                return
 
-        if self.password_function:
+            elif 'COOKIE' in methods:
+                txtorlog.msg("Using COOKIE authentication",
+                             cookiefile, len(self._cookie_data), "bytes")
+                d = self.authenticate(self._cookie_data)
+                d.addCallback(self._bootstrap)
+                d.addErrback(self._auth_failed)
+                return
+
+        if self.password_function and 'HASHEDPASSWORD' in methods:
             d = defer.maybeDeferred(self.password_function)
             d.addCallback(self._do_password_authentication)
             d.addErrback(self._auth_failed)
             return
 
+        if 'NULL' in methods:
+            d = self.queue_command('AUTHENTICATE')
+            d.addCallback(self._bootstrap)
+            d.addErrback(self._auth_failed)
+            return
+
         raise RuntimeError(
             "The Tor I connected to doesn't support SAFECOOKIE nor COOKIE"
             " authentication and I have no password_function specified."
